#!perl -T

use Test::More tests => 1;

#BEGIN {
#    use_ok( 'RT::Condition::NotStartedInBusinessHours' );
#}
#
#diag( "Testing RT::Condition::NotStartedInBusinessHours $RT::Condition::NotStartedInBusinessHours::VERSION, Perl $], $^X" );

ok('TODO nothing testet');
